import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class Main extends Frame {
    Main(String filename) {
        super("CropImage Example");
        try {
            Image fileImage = getToolkit().getImage(filename);

            // Use a media tracker object to wait until all the pixels 
            // have been retrieved.
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(fileImage, 0);
            tracker.waitForID(0);

            // Now copy the retrieved image to an offscreen image.
            addNotify();    // otherwise the following code will fail.
            Image image = createImage(fileImage.getWidth(this), 
                                      fileImage.getHeight(this));
            Graphics g = image.getGraphics();
            g.drawImage(fileImage, 0, 0, this);
            g.dispose();

            add(new ImageCanvas(image), BorderLayout.CENTER);
        } catch (Exception e) {
            e.printStackTrace();
        }
        setSize(300, 300);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    Image image;
    Rectangle selected = new Rectangle();
    Image cropImage;
    Rectangle cropImageRect = new Rectangle();

    ImageCanvas(Image image) {
        this.image = image;

        // Add listeners for mouse and key events
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());
        addKeyListener(new KeyEventHandler());
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
	g.setClip(0, 0, image.getWidth(null), image.getHeight(null));
        g.drawImage(image, 0, 0, this);
        if (cropImage != null) {
            g.drawImage(cropImage, cropImageRect.x, cropImageRect.y, this);
        } else if (!selected.isEmpty()) {
            g.setXORMode(getBackground());
            g.fillRect(selected.x, selected.y, 
                       selected.width, selected.height);
            g.setPaintMode();
        }
    }

    // Convenience method.
    void repaintRect(Rectangle r) {
        repaint(r.x, r.y, r.width, r.height);
    }

    // Event handlers
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (cropImage != null) {
                // Add cropImage to main image.
                Graphics g = image.getGraphics();
                g.drawImage(cropImage, cropImageRect.x, cropImageRect.y, 
                            ImageCanvas.this);
                g.dispose();
                cropImage = null;
            } 
            repaintRect(selected);
            selected.setLocation(evt.getX(), evt.getY());
            repaintRect(selected);
        }

        public void mouseReleased(MouseEvent evt) {
            repaintRect(selected);
            selected.width = Math.max(0, evt.getX() - selected.x);
            selected.height = Math.max(0, evt.getY() - selected.y);
            repaintRect(selected);
        }
    }

    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            repaintRect(selected);
            selected.width = Math.max(0, evt.getX() - selected.x);
            selected.height = Math.max(0, evt.getY() - selected.y);
            repaintRect(selected);
        }

        public void mouseMoved(MouseEvent evt) {
            if (cropImage != null) {
                // Move the cropped image around.
                repaintRect(cropImageRect);
                cropImageRect.setLocation(evt.getX(), evt.getY());
                repaintRect(cropImageRect);
            }
        }
    }

    class KeyEventHandler extends KeyAdapter {
        public void keyPressed(KeyEvent evt) {
            char key = evt.getKeyChar();
            if (key == 'c') {
                if (!selected.isEmpty()) {
                    // Create crop image using a CropImageFilter.
                    CropImageFilter imgf = 
                        new CropImageFilter(selected.x, selected.y, 
                                            selected.width, selected.height);
                    ImageProducer ip = image.getSource();
                    ip = new FilteredImageSource(ip, imgf);
                    cropImage = getToolkit().createImage(ip);
                    cropImageRect = new Rectangle(selected.x, selected.y, 
                                                  selected.width, 
                                                  selected.height);
                }
            } else if (cropImage != null) {
                repaintRect(cropImageRect);
                cropImage = null;
            }
            // Delete the selection.
            repaintRect(selected);
            selected.width = selected.height = 0;
        }
    }
}
