import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    Main() {
        super("LayoutManager2 Example");
        setLayout(new TextFlowLayout());

        // Add a few of our special labels.
        for (int i=0; i<3; i++) {
            add(new LabelComponent("java", 20+2*i));
        }

        // Now add a container with a few of our special labels.
        Panel p = new Panel(new TextFlowLayout());
        p.setBackground(Color.green);
        for (int i=0; i<3; i++) {
            p.add(new LabelComponent("java", 30+2*i));
        }
        add(p);

        // Add a few more special labels.
        for (int i=0; i<3; i++) {
            add(new LabelComponent("java", 10+2*i));
        }

        pack();
        show();
    }

    public static void main(String[] args) {
        new Main();
    }
}

class TextFlowLayout implements LayoutManager2 {
    // If < 0, the layout is invalid.
    int maxAscent = -1;
    int maxDescent = 0;

    // type=0->minimum, type=1->preferred, type=2->maximum.
    public Dimension layoutSize(Container cont, int type) {
        Insets insets = cont.getInsets();
        int w = 0;

        // Grab tree lock to make sure components are not removed while
        // getting their sizes
        synchronized (cont.getTreeLock()) {
            Dimension d = new Dimension(0, 0);
            maxAscent = 0;
            maxDescent = 0;

            for (int i=0; i<cont.getComponentCount(); i++) {
                Component c = cont.getComponent(i);

                switch (type) {
                  case 0:
                    d = c.getMinimumSize();
                    break;
                  case 1:
                    d = c.getPreferredSize();
                    break;
                  case 2:
                    d = c.getMaximumSize();
                    break;
                }

                int a = (int)(c.getAlignmentY()*d.height);
                maxAscent = Math.max(maxAscent, a);
                maxDescent = Math.max(maxDescent, d.height-a);
                w += d.width;
            }
        }
        return new Dimension(insets.left + insets.right + w,
             insets.top + insets.bottom + maxAscent+maxDescent);
    }
    
    public Dimension minimumLayoutSize(Container cont) {
        return layoutSize(cont, 0);
    }
    public Dimension preferredLayoutSize(Container cont) {
        return layoutSize(cont, 1);
    }
    public Dimension maximumLayoutSize(Container cont) {
        return layoutSize(cont, 2);
    }

    public void layoutContainer(Container cont) {
        Insets insets = cont.getInsets();
        if (maxAscent < 0) {
            layoutSize(cont, 1);
        }
        
        // Grab tree lock to make sure components are not removed while
        // laying them out.
        synchronized (cont.getTreeLock()) {
            int x = insets.left;

            for (int i=0; i<cont.getComponentCount(); i++) {
                Component c = cont.getComponent(i);
                Dimension d = c.getPreferredSize();

                c.setBounds(x, 
                    insets.top + (int)(maxAscent-d.height*c.getAlignmentY()),
                    d.width, d.height);
                x += d.width;
            }
        }
    }

    public float getLayoutAlignmentX(Container cont) {
        return 0.5f;
    }
    public float getLayoutAlignmentY(Container cont) {
        if (maxAscent < 0) {
            layoutSize(cont, 1);
        }
        if (maxAscent > 0) {
            return (float)maxAscent / (float)(maxAscent+maxDescent);
        }
        return 0.5f;
    }
    public void invalidateLayout(Container cont) {
        maxAscent = -1;
    }

    // These methods are not used.
    public void addLayoutComponent(String name, Component comp) {
    }
    public void addLayoutComponent(Component comp, Object constraints) {
    }
    public void removeLayoutComponent(Component comp) {
    }
}

class LabelComponent extends Canvas {
    String label;
    Font font;
    FontMetrics fontM;
    boolean hasFocus = false;
    int curAlignment = 0;
    float[] alignments = {-1.0f, //   baseline alignment
	Component.CENTER_ALIGNMENT,
	Component.TOP_ALIGNMENT,
	Component.BOTTOM_ALIGNMENT};

    LabelComponent(String label, int size) {
        this.label = label;
        font = new Font("Serif", Font.PLAIN, size);
        fontM = getFontMetrics(font);

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());
        addFocusListener(new FocusEventHandler());
    }

    public float getAlignmentY() {
        if (alignments[curAlignment] == -1.0f) {
            return (float)fontM.getAscent() / (float)fontM.getHeight();
        } else {
            return alignments[curAlignment];
        }
    }
        
    public Dimension getPreferredSize() {
        int space = fontM.charWidth(' ');
        return new Dimension(space + fontM.stringWidth(label),
            fontM.getHeight());
    }

    public void paint(Graphics g) {
        if (hasFocus) {
            g.setColor(Color.pink);
        } else {
            g.setColor(Color.lightGray);
        }
        g.fillRect(0, 0, getSize().width, getSize().height);
        g.setColor(Color.black);

        // The following isn't necessary in a native component.
        g.setFont(font);
        FontMetrics fontM = g.getFontMetrics();

        g.drawString(label, fontM.charWidth(' ')/2, fontM.getAscent());

        // Draw baseline.
        g.drawLine(0, fontM.getAscent(), getSize().width, fontM.getAscent());
    }

    public boolean isFocusTraversable() {
        return true;
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            int inc = 0;

            if (evt.isShiftDown()) {
                // Change the alignment.
                curAlignment = (curAlignment + 1) % alignments.length;
            } else if ((evt.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
                // Decrease the font size.
                if (font.getSize() > 6) {
                    inc = -1;
                }
            } else {
                // Increase the font size.
                inc = 1;
            }

            font = new Font(font.getFamily(), font.getStyle(), 
                font.getSize() + inc);
            fontM = getFontMetrics(font);

            // Invalidate current component and validate the entire frame.
            invalidate();

            // Find the top-level window.
            Component c = getParent();
            while (c != null) {
                if (c instanceof Window) {
                    break;
                }
                c = c.getParent();
            }

            // Validate the whole tree.
            if (c != null) {
                c.validate();
            }
            requestFocus();
            repaint();
        }
    }
    class FocusEventHandler extends FocusAdapter {
        public void focusGained(FocusEvent evt) {
            hasFocus = true;
            repaint();
        }
        public void focusLost(FocusEvent evt) {
            hasFocus = false;
            repaint();
        }
    }
}